==> kleinod-0.23.3.37-sources.jar: de/pfabulist/kleinod/collection/P.java <==
package de.pfabulist.kleinod.collection;

import java.util.function.BiFunction;
import java.util.function.Function;

/**
 * ** BEGIN LICENSE BLOCK *****
 * BSD License (2 clause)
 * Copyright (c) 2006 - 2014, Stephan Pfab
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Stephan Pfab BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * **** END LICENSE BLOCK ****
 */
public class P<A,B> {

    public final A i0; // NOSONAR
    public final B i1; // NOSONAR

    public P(A i0, B i1) {
        this.i0 = i0;
        this.i1 = i1;
    }

    public static  <A,B> P<A,B> of(A a, B b) {
        return new P<>(a,b);
    }

    public <R> R with( BiFunction<A,B,R> f ) {
        return f.apply( i0, i1 );
    }
    
    public static <R> R until( R a, Function<R,P<Boolean,R>> f ) {
        
        while(true) {
            P<Boolean, R> res = f.apply(a);
            if ( res.i0 ) {
                return res.i1;
            }
            
            a = res.i1;
        }
    }



}

==> kleinod-0.24-sources.jar: de/pfabulist/kleinod/collection/P.java <==
package de.pfabulist.kleinod.collection;

import java.util.function.BiFunction;
import java.util.function.Function;

/**
 * ** BEGIN LICENSE BLOCK *****
 * BSD License (2 clause)
 * Copyright (c) 2006 - 2015, Stephan Pfab
 * All rights reserved.
 * <p>
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * <p>
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL Stephan Pfab BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * **** END LICENSE BLOCK ****
 */
public class P<A, B> {

    public final A i0; // NOSONAR
    public final B i1; // NOSONAR

    public P( A i0, B i1 ) {
        this.i0 = i0;
        this.i1 = i1;
    }

    public static <A, B> P<A, B> of( A a, B b ) {
        return new P<>( a, b );
    }

    public <R> R with( BiFunction<A, B, R> f ) {
        return f.apply( i0, i1 );
    }

    public static <R> R until( R a, Function<R, P<Boolean, R>> f ) {

        R val = a;

        while( true ) {
            P<Boolean, R> res = f.apply( val );
            if( res.i0 ) {
                return res.i1;
            }

            val = res.i1;
        }
    }

}
